@@ -1,25 +1,8 @@
-######################################################################
-# Copyright (C) 2011,2012,2014 Jaakko Luttinen
+################################################################################
+# Copyright (C) 2011-2012,2014 Jaakko Luttinen
 #
-# This file is licensed under Version 3.0 of the GNU General Public
-# License. See LICENSE for a text of the license.
-######################################################################
-
-######################################################################
-# This file is part of BayesPy.
-#
-# BayesPy is free software: you can redistribute it and/or modify it
-# under the terms of the GNU General Public License version 3 as
-# published by the Free Software Foundation.
-#
-# BayesPy is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with BayesPy.  If not, see <http://www.gnu.org/licenses/>.
-######################################################################
+# This file is licensed under the MIT License.
+################################################################################
 
 """
 Module for the categorical distribution node.
@@ -44,15 +27,6 @@ class CategoricalMoments(MultinomialMoments):
     Class for the moments of categorical variables.
     """
 
-    
-    def __init__(self, categories):
-        """
-        Create moments object for categorical variables
-        """
-        self.D = categories
-        super().__init__()
-
-        
     def compute_fixed_moments(self, x):
         """
         Compute the moments for a fixed value
@@ -62,29 +36,45 @@ class CategoricalMoments(MultinomialMoments):
         x = np.asanyarray(x)
         if not misc.isinteger(x):
             raise ValueError("Values must be integers")
-        if np.any(x < 0) or np.any(x >= self.D):
+        if np.any(x < 0) or np.any(x >= self.categories):
             raise ValueError("Invalid category index")
 
-        u0 = np.zeros((np.size(x), self.D))
-        u0[[np.arange(np.size(x)), np.ravel(x)]] = 1
-        u0 = np.reshape(u0, np.shape(x) + (self.D,))
+        u0 = np.zeros((np.size(x), self.categories))
+        u0[(np.arange(np.size(x)), np.ravel(x))] = 1
+        u0 = np.reshape(u0, np.shape(x) + (self.categories,))
 
         return [u0]
-    
 
-    def compute_dims_from_values(self, x):
+
+    @classmethod
+    def from_values(cls, x, categories):
         """
         Return the shape of the moments for a fixed value.
 
         The observations are scalar.
         """
+        return cls(categories)
+        raise DeprecationWarning()
         return ( (self.D,), )
 
 
+    def get_instance_conversion_kwargs(self):
+        return dict(categories=self.categories)
+
+
+    def get_instance_converter(self, categories):
+        if categories is not None and categories != self.categories:
+            raise ValueError(
+                "No automatic conversion from CategoricalMoments to "
+                "CategoricalMoments with different number of categories"
+            )
+        return None
+
+
 class CategoricalDistribution(MultinomialDistribution):
     """
     Class for the VMP formulas of categorical variables.
-    """    
+    """
 
     def __init__(self, categories):
         """
@@ -98,35 +88,7 @@ class CategoricalDistribution(MultinomialDistribution):
             raise ValueError("Number of categoriess must be non-negative")
         self.D = categories
         super().__init__(1)
-        
 
-    def compute_message_to_parent(self, parent, index, u, u_p):
-        """
-        Compute the message to a parent node.
-        """
-        return super().compute_message_to_parent(parent, index, u, u_p)
-
-
-    def compute_phi_from_parents(self, u_p, mask=True):
-        """
-        Compute the natural parameter vector given parent moments.
-        """
-        return super().compute_phi_from_parents(u_p, mask=mask)
-
-
-    def compute_moments_and_cgf(self, phi, mask=True):
-        """
-        Compute the moments and :math:`g(\phi)`.
-        """
-        return super().compute_moments_and_cgf(phi, mask=mask)
-
-        
-    def compute_cgf_from_parents(self, u_p):
-        """
-        Compute :math:`\mathrm{E}_{q(p)}[g(p)]`
-        """
-        return super().compute_cgf_from_parents(u_p)
-    
 
     def compute_fixed_moments_and_f(self, x, mask=True):
         """
@@ -142,7 +104,7 @@ class CategoricalDistribution(MultinomialDistribution):
 
         # Form a binary matrix with only one non-zero (1) in the last axis
         u0 = np.zeros((np.size(x), self.D))
-        u0[[np.arange(np.size(x)), np.ravel(x)]] = 1
+        u0[(np.arange(np.size(x)), np.ravel(x))] = 1
         u0 = np.reshape(u0, np.shape(x) + (self.D,))
         u = [u0]
 
@@ -151,7 +113,7 @@ class CategoricalDistribution(MultinomialDistribution):
 
         return (u, f)
 
-    
+
     def random(self, *phi, plates=None):
         """
         Draw a random sample from the distribution.
@@ -161,23 +123,23 @@ class CategoricalDistribution(MultinomialDistribution):
         p = np.exp(logp)
         return random.categorical(p, size=plates)
 
-    
+
 class Categorical(ExponentialFamily):
     r"""
     Node for categorical random variables.
-    
+
     The node models a categorical random variable :math:`x \in \{0,\ldots,K-1\}`
     with prior probabilities :math:`\{p_0, \ldots, p_{K-1}\}` for each category:
-    
+
     .. math::
 
         p(x=k) = p_k \quad \text{for } k\in \{0,\ldots,K-1\}.
 
     Parameters
     ----------
-    
+
     p : Dirichlet-like node or (...,K)-array
-    
+
         Probabilities for each category
 
     See also
@@ -186,9 +148,6 @@ class Categorical(ExponentialFamily):
     """
 
 
-    _parent_moments = [DirichletMoments()]
-
-
     def __init__(self, p, **kwargs):
         """
         Create Categorical node.
@@ -197,40 +156,43 @@ class Categorical(ExponentialFamily):
 
 
     @classmethod
-    @ensureparents
     def _constructor(cls, p, **kwargs):
         """
         Constructs distribution and moments objects.
 
         This method is called if useconstructor decorator is used for __init__.
-        
+
         Becase the distribution and moments object depend on the number of
         categories, that is, they depend on the parent node, this method can be
         used to construct those objects.
         """
 
         # Get the number of categories
+        p = cls._ensure_moments(p, DirichletMoments)
         D = p.dims[0][0]
 
+        parent_moments = (p._moments,)
+
         parents = [p]
-        moments = CategoricalMoments(D)
         distribution = CategoricalDistribution(D)
+        moments = CategoricalMoments(D)
 
         return (parents,
                 kwargs,
-                ( (D,), ),
+                moments.dims,
                 cls._total_plates(kwargs.get('plates'),
                                   distribution.plates_from_parent(0, p.plates)),
-                distribution, 
-                moments, 
-                cls._parent_moments)
-    
+                distribution,
+                moments,
+                parent_moments)
 
-    def show(self):
+
+    def __str__(self):
         """
         Print the distribution using standard parameterization.
         """
         p = self.u[0]
-        print("%s ~ Categorical(p)" % self.name)
-        print("  p = ")
-        print(p)
+        return ("%s ~ Categorical(p)\n"
+                "  p = \n"
+                "%s\n"
+                % (self.name, p))
